import re
import sys

from .output import print_table


def raw_input_no_history(prompt='', stream=None, _input=None):
    # A raw_input() replacement that doesn't save the string in the
    # GNU readline history.
    if not stream:
        stream = sys.stderr
    if not _input:
        _input = sys.stdin
    prompt = str(prompt)
    if prompt:
        stream.write(prompt)
        stream.flush()
    line = _input.readline()
    if not line:
        raise EOFError
    if line[-1] == '\n':
        line = line[:-1]
    return line


def get_choices(records, fields, labels=None, intro=None, prompt=''):
    choice_labels = []
    if labels:
        choice_labels = labels[:]
        choice_labels.insert(0, '#')

    choices = []
    for index, record in enumerate(records):
        row = [str(index)]
        for field in fields:
            row.append(record[field])
        choices.append(row)

    if intro:
        print()
        print(intro)
    print()
    print_table(choices, labels=choice_labels)
    choice = raw_input_no_history(prompt)
    if choice:
        return choice
    else:
        return get_choices(records, fields, labels=labels, intro=intro, prompt=prompt)


def choose_one(records, fields, labels=None, intro='Available choices:', prompt='Choose one by #: '):
    if len(records) == 1:
        return records[0]

    choices = get_choices(records, fields, labels=labels, intro=intro, prompt=prompt)

    try:
        choosen_index = int(choices)
        if choosen_index >= len(records):
            raise ValueError()
    except ValueError:
        print('{} is not a valid choice'.format(choices))
        return choose_one(records, fields, labels=labels, intro=intro)
    else:
        return records[choosen_index]


def choose_many(records, fields, labels=None, intro='Available choices:', prompt='Choose one or more by #: '):
    if len(records) == 1:
        return records[0]

    choices = get_choices(records, fields, labels=labels, intro=intro, prompt=prompt)

    indexes = []
    for current_index in re.split('[, ]', choices):
        try:
            current_index = int(current_index)
            if current_index >= len(records):
                raise ValueError()
            indexes.append(current_index)
        except ValueError:
            print('{} is not a valid choice'.format(current_index))
            return choose_one(records, fields, labels=labels, intro=intro)

    return [records[index] for index in indexes]
