#!/usr/bin/env python3

import sys
import argparse
import logging

logging.basicConfig(level=logging.INFO, format='%(levelname)s: %(message)s', stream=sys.stderr)

log = logging.getLogger(__name__)


def main():
    args = parse_args(sys.argv[1:])
    parts = args.room.split('/')
    data = {}
    if len(parts) >= 1:
        data['building'] = parts[0]
    if len(parts) >= 2:
        data['floor'] = parts[1]
    if len(parts) >= 2:
        data['room'] = parts[2]

    if args.location:
        print('http://www.mapsearch.ethz.ch/map.do?gebaeudeMap={building}'.format(**data))
    if args.floor_plan:
        print('http://www.rauminfo.ethz.ch/grundrissplan.gif?gebaeude={building}&geschoss={floor}&raumNr={room}'.format(
            **data))


def parse_args(argv):
    parser = argparse.ArgumentParser(description='''Construct URI\'s to ETHZ room plans and print them to stdout.

usage example:
  firefox $(roominfo -f -l CAB/H/88)
''', formatter_class=argparse.RawTextHelpFormatter)
    parser.add_argument('-d', '--debug', action='store_true', default=False,
                        help='set log level to debug')
    parser.add_argument('-v', '--verbose', action='store_true', default=False,
                        help='be verbose, set log level to info')
    parser.add_argument('-f', '--floor-plan', action='store_true', default=False,
                        help='print floor plan uri')
    parser.add_argument('-l', '--location', action='store_true', default=False,
                        help='print location of building uri')
    parser.add_argument('room',
                        nargs=1,
                        help='name of the room in the format: CAB/H/88')

    args = parser.parse_args(argv)
    args.room = args.room[0]
    log.debug(args)
    if args.verbose:
        logging.root.setLevel(logging.INFO)
    # debug overrides verbose
    if args.debug:
        logging.root.setLevel(logging.DEBUG)

    log.debug(args)
    return args


if __name__ == '__main__':
    main()
