from sans.ethz import Error


class AmbiguousNameError(Error):
    def __init__(self, token, identifiers, label='name'):
        msg = 'Ambiguous {} \'{}\' - could be either of {}'.format(label, token, ', '.join(identifiers))
        super(AmbiguousNameError, self).__init__(msg)


class UnknownNameError(Error):
    def __init__(self, token, label='name'):
        msg = 'Unknown {} \'{}\''.format(label, token)
        super(UnknownNameError, self).__init__(msg)


def get_canonical_name(token, choices, label=None):
    """get_canonical_name(token <string>, choices <list>) -> string

    Performes a case insensitive match of token against choices.
    """
    if token in choices:
        # exact match
        return token

    names = []
    token_lower = token.lower()
    for name in choices:
        if name.lower().startswith(token_lower):
            names.append(name)
    if len(names) > 1:
        raise AmbiguousNameError(token, names, label=label)
    else:
        if names:
            return names[0]
        else:
            raise UnknownNameError(token, label=label)


def parse_properties(args, property_names, label='property'):
    properties = {}
    for arg in args[:]:
        if ':' in arg:
            name, value = arg.split(':', 1)
            name = get_canonical_name(name, property_names, label=label)
            if name:
                properties[name] = value
                args.remove(arg)
    return properties
