import logging

import httplib2

from sans.ethz import util


class UnsecureURIError(httplib2.HttpLib2Error):
    pass


class Http(httplib2.Http):
    def request(self, uri, **kwargs):
        """Accepts an additional boolean argument 'auth_without_challenge' and if given
        adds http authorization headers to the request even if not explicitly challenged
        for by the server.
        
        This is a workaround for brain dead server implementations which never send HTTP 
        authentication challenges, but accept unsolicited auth info.
        """
        auth_without_challenge = kwargs.pop('auth_without_challenge', False)
        if auth_without_challenge:
            uri = httplib2.iri2uri(uri)
            (scheme, host, request_uri, defrag_uri) = httplib2.urlnorm(uri)

            if scheme != 'https':
                raise UnsecureURIError('Refusing to send credentials to unsecure uri: %s' % uri)

            default_cred = None
            cred = None
            for (domain, name, password) in self.credentials.credentials:
                if host == domain:
                    cred = (name, password)
                    break
                if domain == '':
                    default_cred = (name, password)
            if not cred and default_cred:
                cred = default_cred

            if cred:
                # add basic authorization to http headers
                import base64
                headers = kwargs.setdefault('headers', {})
                headers['authorization'] = 'Basic ' + base64.encodebytes(
                    "{}:{}".format(*cred).strip().encode()).decode()
        return super(Http, self).request(uri, **kwargs)


def get_http_client(username=None, password=None, host=None):
    http = Http()
    if not username or not password:
        username, password = util.get_username_password(username=username, host=host)
    logging.debug('get_http_client: username={0}, host={1}'.format(username, host))
    http.add_credentials(username, password, host)
    return http
