import logging
import sys

try:
    import ldap
except ImportError:
    print('You need to install the python ldap bindings to use this command.')
    print('On Gentoo run: /usr/bin/emerge python3-ldap -va')
    print('On debian run: apt-get install python3-ldap')
    sys.exit(1)


def set_ldap_options(options):
    """Set global ldap options"""
    for k, v in options.items():
        if hasattr(ldap, k):
            ldap.set_option(getattr(ldap, k), v)


def ldap_search_async(l, basedn, scope, _filter, attributes, timeout):
    result_id = l.search(basedn, scope, _filter, attributes)
    logging.debug('result_id: %s', result_id)
    while True:
        result_type, result_data = l.result(result_id, timeout)
        logging.debug('result_type: %s', result_type)
        logging.debug('result_data: %s', result_data)

        if not result_data:
            break
        else:
            if result_type == ldap.RES_SEARCH_ENTRY:
                yield result_data


def ldap_search(url, binddn, bindpw, basedn, scope=ldap.SCOPE_SUBTREE, _filter='(objectclass=*)', attributes=None,
                _async=False, timeout=0):
    try:
        logging.debug('Connecting to: %s', url)
        l = ldap.initialize(url)
        l.simple_bind_s(binddn, bindpw)

        logging.debug('Searching..')

        try:
            if _async:
                return ldap_search_async(l, basedn, scope, _filter, attributes, timeout)
            else:
                return l.search_s(basedn, scope, _filter, attributes)

        except ldap.LDAPError as error_message:
            logging.error('Search failed: {}'.format(error_message))

    except ldap.LDAPError as error_message:
        logging.error('Connection failed: {}'.format(error_message))
        raise


def dict_from_ldap_result(data, attributes=None):
    if attributes:
        d = dict.fromkeys(attributes)
    else:
        d = {}
    d['dn'] = data[0]
    for k, v in data[1].items():
        if len(v) == 1:
            d[k] = v[0].decode()
        else:
            d[k] = v
    return d
